import chess

PIECE_SYMBOLS = {
    chess.KING: "K",
    chess.QUEEN: "Q",
    chess.ROOK: "R",
    chess.BISHOP: "B",
    chess.KNIGHT: "N",
    chess.PAWN: "",
}

THEMES = {
    # Tactics
    "double_check": "Double Check",
    "discovered_check": "Discovered Check",
    "fork": "Fork",
    "pin": "Pin",
    "skewer": "Skewer",
    "trapped_piece": "Trapped Piece",
    "hanging_capture": "Hanging Capture",
    "deflection": "Deflection",
    "decoy": "Decoy",
"removing_defender_material": "Removing Defender (winning material)",
    "removing_defender_mate": "Removing Defender (mate threat)",
    "exploiting_pin": "Exploiting Pin",
    "zwischenzug": "Zwischenzug",
    "discovered_attack": "Discovered Attack",
    "double_attack": "Double Attack",

    # Strategy
    "weak_square": "Weak Square",
    "open_file": "Open File",
    "outpost": "Outpost",
    "bad_bishop": "Bad Bishop",
    "isolated_pawn": "Isolated Pawn",
    "doubled_pawns": "Doubled Pawns",
    "center_control": "Center Control",

    # Checkmate patterns
    "back_rank_mate": "Back Rank Mate",
    "smothered_mate": "Smothered Mate",
    "legal_mate": "Légal's Mate",
    "anastasia_mate": "Anastasia's Mate",
    "arabian_mate": "Arabian Mate",
    "two_bishops_mate": "Two Bishops Mate",
    "epaulette_mate": "Epaulette Mate",
    "boden_mate": "Boden's Mate",
    "damiano_mate": "Damiano's Mate",
    "greco_mate": "Greco's Mate",
}
