import chess

PIECE_SYMBOLS = {
    chess.KING: "Кр",
    chess.QUEEN: "Ф",
    chess.ROOK: "Л",
    chess.BISHOP: "С",
    chess.KNIGHT: "К",
    chess.PAWN: "",
}

THEMES = {
    # Тактика
    "double_check": "Двойной шах",
    "discovered_check": "Вскрытый шах",
    "fork": "Вилка",
    "pin": "Связка",
    "skewer": "Копьё",
    "trapped_piece": "Пойманная фигура",
    "hanging_capture": "Взятие висящей фигуры",
    "deflection": "Отвлечение",
    "decoy": "Завлечение",
"removing_defender_material": "Уничтожение защитника (выигрыш фигуры)",
    "removing_defender_mate": "Уничтожение защитника (угроза мата)",
    "exploiting_pin": "Использование связки",
    "zwischenzug": "Промежуточный ход",
    "discovered_attack": "Вскрытая атака",
    "double_attack": "Двойная атака",

    # Стратегия
    "weak_square": "Слабое поле",
    "open_file": "Открытая линия",
    "outpost": "Форпост",
    "bad_bishop": "Плохой слон",
    "isolated_pawn": "Изолированная пешка",
    "doubled_pawns": "Сдвоенные пешки",
    "center_control": "Контроль центра",

    # Мат
    "back_rank_mate": "Линейный мат",
    "smothered_mate": "Спёртый мат",
    "legal_mate": "Мат Легаля",
    "anastasia_mate": "Мат Анастасии",
    "arabian_mate": "Арабский мат",
    "two_bishops_mate": "Мат двумя слонами",
    "epaulette_mate": "Эполетный мат",
    "boden_mate": "Мат Бодена",
    "damiano_mate": "Мат Дамиано",
    "greco_mate": "Мат Греко",
}
