from .base import BaseDetector
from .tactics import (
    DoubleCheckDetector,
    ForkDetector,
    DiscoveredCheckDetector,
    PinDetector,
    SkewerDetector,
    TrappedPieceDetector,
    HangingCaptureDetector,
    RemovingDefenderMaterialDetector,
    RemovingDefenderMateDetector,
    ExploitingPinDetector,
)

__all__ = [
    "BaseDetector",
    "DoubleCheckDetector",
    "ForkDetector",
    "DiscoveredCheckDetector",
    "PinDetector",
    "SkewerDetector",
    "TrappedPieceDetector",
    "HangingCaptureDetector",
    "RemovingDefenderMaterialDetector",
    "RemovingDefenderMateDetector",
    "ExploitingPinDetector",
]
